@@ -36,7 +36,7 @@ class MetaPandasDataset(Dataset):
     """
 
     def __init__(self, *args, **kwargs):
-        super(MetaPandasDataset, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @classmethod
     def column_map_expectation(cls, func):
@@ -55,15 +55,28 @@ class MetaPandasDataset(Dataset):
         @cls.expectation(argspec)
         @wraps(func)
         def inner_wrapper(
-            self, column, mostly=None, result_format=None, *args, **kwargs
+            self,
+            column,
+            mostly=None,
+            result_format=None,
+            row_condition=None,
+            condition_parser=None,
+            *args,
+            **kwargs
         ):
 
             if result_format is None:
                 result_format = self.default_expectation_args["result_format"]
 
             result_format = parse_result_format(result_format)
+            if row_condition and self._supports_row_condition:
+                data = self._apply_row_condition(
+                    row_condition=row_condition, condition_parser=condition_parser
+                )
+            else:
+                data = self
 
-            series = self[column]
+            series = data[column]
             if func.__name__ in [
                 "expect_column_values_to_not_be_null",
                 "expect_column_values_to_be_null",
@@ -161,6 +174,8 @@ class MetaPandasDataset(Dataset):
             mostly=None,
             ignore_row_if="both_values_are_missing",
             result_format=None,
+            row_condition=None,
+            condition_parser=None,
             *args,
             **kwargs
         ):
@@ -168,6 +183,9 @@ class MetaPandasDataset(Dataset):
             if result_format is None:
                 result_format = self.default_expectation_args["result_format"]
 
+            if row_condition:
+                self = self.query(row_condition).reset_index(drop=True)
+
             series_A = self[column_A]
             series_B = self[column_B]
 
@@ -260,6 +278,8 @@ class MetaPandasDataset(Dataset):
             mostly=None,
             ignore_row_if="all_values_are_missing",
             result_format=None,
+            row_condition=None,
+            condition_parser=None,
             *args,
             **kwargs
         ):
@@ -267,6 +287,9 @@ class MetaPandasDataset(Dataset):
             if result_format is None:
                 result_format = self.default_expectation_args["result_format"]
 
+            if row_condition:
+                self = self.query(row_condition).reset_index(drop=True)
+
             test_df = self[column_list]
 
             if ignore_row_if == "all_values_are_missing":
@@ -314,16 +337,36 @@ class MetaPandasDataset(Dataset):
 
 class PandasDataset(MetaPandasDataset, pd.DataFrame):
     """
-    PandasDataset instantiates the great_expectations Expectations API as a subclass of a pandas.DataFrame.
+PandasDataset instantiates the great_expectations Expectations API as a subclass of a pandas.DataFrame.
 
-    For the full API reference, please see :func:`Dataset <great_expectations.data_asset.dataset.Dataset>`
+For the full API reference, please see :func:`Dataset <great_expectations.data_asset.dataset.Dataset>`
 
-    Notes:
-        1. Samples and Subsets of PandaDataSet have ALL the expectations of the original \
-           data frame unless the user specifies the ``discard_subset_failing_expectations = True`` \
-           property on the original data frame.
-        2. Concatenations, joins, and merges of PandaDataSets contain NO expectations (since no autoinspection
-           is performed by default).
+Notes:
+    1. Samples and Subsets of PandaDataSet have ALL the expectations of the original \
+       data frame unless the user specifies the ``discard_subset_failing_expectations = True`` \
+       property on the original data frame.
+    2. Concatenations, joins, and merges of PandaDataSets contain NO expectations (since no autoinspection
+       is performed by default).
+
+--ge-feature-maturity-info--
+
+    id: validation_engine_pandas
+    title: Validation Engine - Pandas
+    icon:
+    short_description: Use Pandas DataFrame to validate data
+    description: Use Pandas DataFrame to validate data
+    how_to_guide_url:
+    maturity: Production
+    maturity_details:
+        api_stability: Stable
+        implementation_completeness: Complete
+        unit_test_coverage: Complete
+        integration_infrastructure_test_coverage: N/A -> see relevant Datasource evaluation
+        documentation_completeness: Complete
+        bug_risk: Low
+        expectation_completeness: Complete
+
+--ge-feature-maturity-info--
     """
 
     # this is necessary to subclass pandas in a proper way.
@@ -335,6 +378,7 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
     _internal_names = pd.DataFrame._internal_names + [
         "_batch_kwargs",
         "_batch_markers",
+        "_batch_parameters",
         "_batch_id",
         "_expectation_suite",
         "_config",
@@ -343,6 +387,7 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         "discard_subset_failing_expectations",
     ]
     _internal_names_set = set(_internal_names)
+    _supports_row_condition = True
 
     # We may want to expand or alter support for subclassing dataframes in the future:
     # See http://pandas.pydata.org/pandas-docs/stable/extending.html#extending-subclassing-pandas
@@ -363,15 +408,26 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
             )
             if self.discard_subset_failing_expectations:
                 self.discard_failing_expectations()
-        super(PandasDataset, self).__finalize__(other, method, **kwargs)
+        super().__finalize__(other, method, **kwargs)
         return self
 
     def __init__(self, *args, **kwargs):
-        super(PandasDataset, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.discard_subset_failing_expectations = kwargs.get(
             "discard_subset_failing_expectations", False
         )
 
+    def _apply_row_condition(self, row_condition, condition_parser):
+        if condition_parser not in ["python", "pandas"]:
+            raise ValueError(
+                "condition_parser is required when setting a row_condition,"
+                " and must be 'python' or 'pandas'"
+            )
+        else:
+            return self.query(row_condition, parser=condition_parser).reset_index(
+                drop=True
+            )
+
     def get_row_count(self):
         return self.shape[0]
 
@@ -471,6 +527,57 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
                 result = result[result <= max_val]
         return len(result)
 
+    def get_crosstab(
+        self,
+        column_A,
+        column_B,
+        bins_A=None,
+        bins_B=None,
+        n_bins_A=None,
+        n_bins_B=None,
+    ):
+        """Get crosstab of column_A and column_B, binning values if necessary"""
+        series_A = self.get_binned_values(self[column_A], bins_A, n_bins_A)
+        series_B = self.get_binned_values(self[column_B], bins_B, n_bins_B)
+        return pd.crosstab(series_A, columns=series_B)
+
+    def get_binned_values(self, series, bins, n_bins):
+        """
+        Get binned values of series.
+
+        Args:
+            Series (pd.Series): Input series
+            bins (list):
+                Bins for the series. List of numeric if series is numeric or list of list
+                of series values else.
+            n_bins (int): Number of bins. Ignored if bins is not None.
+        """
+        if n_bins is None:
+            n_bins = 10
+
+        if series.dtype in ["int", "float"]:
+            if bins is None:
+                bins = np.histogram_bin_edges(series[series.notnull()], bins=n_bins)
+                # Make sure max of series is included in rightmost bin
+                bins[-1] = np.nextafter(bins[-1], bins[-1] + 1)
+
+            # Missings get digitized into bin = n_bins+1
+            return np.digitize(series, bins=bins)
+
+        else:
+            if bins is None:
+                value_counts = series.value_counts(sort=True)
+                if len(value_counts) < n_bins + 1:
+                    return series.fillna("(missing)")
+                else:
+                    other_values = sorted(value_counts.index[n_bins:])
+                    replace = {value: "(other)" for value in other_values}
+            else:
+                replace = dict()
+                for x in bins:
+                    replace.update({value: ", ".join(x) for value in x})
+            return series.replace(to_replace=replace).fillna("(missing)")
+
     ### Expectation methods ###
 
     @DocInherit
@@ -480,6 +587,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -494,6 +603,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -509,6 +620,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -525,7 +638,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         # Since we've now received the default arguments *before* the expectation decorator, we need to
         # ensure we only pass what we actually received. Hence, we'll use kwargs
         # mostly=None,
-        # result_format=None, include_config=None, catch_exceptions=None, meta=None
+        # result_format=None,
+        # row_condition=None, condition_parser=None, include_config=None, catch_exceptions=None, meta=None
     ):
         """
         The pandas implementation of this expectation takes kwargs mostly, result_format, include_config,
@@ -565,16 +679,21 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
 
             # First, if there is an existing expectation of this type, delete it. Then change the one we created to be
             # of the proper expectation_type
-            existing_expectations = self.find_expectation_indexes(
-                "expect_column_values_to_be_of_type", column
+            existing_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="expect_column_values_to_be_of_type",
+                    kwargs={"column": column},
+                )
             )
             if len(existing_expectations) == 1:
                 self._expectation_suite.expectations.pop(existing_expectations[0])
 
             # Now, rename the expectation we just added
-
-            new_expectations = self.find_expectation_indexes(
-                "_expect_column_values_to_be_of_type__aggregate", column
+            new_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="_expect_column_values_to_be_of_type__aggregate",
+                    kwargs={"column": column},
+                )
             )
             assert len(new_expectations) == 1
             old_config = self._expectation_suite.expectations[new_expectations[0]]
@@ -596,15 +715,21 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
 
             # First, if there is an existing expectation of this type, delete it. Then change the one we created to be
             # of the proper expectation_type
-            existing_expectations = self.find_expectation_indexes(
-                "expect_column_values_to_be_of_type", column
+            existing_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="expect_column_values_to_be_of_type",
+                    kwargs={"column": column},
+                )
             )
             if len(existing_expectations) == 1:
                 self._expectation_suite.expectations.pop(existing_expectations[0])
 
             # Now, rename the expectation we just added
-            new_expectations = self.find_expectation_indexes(
-                "_expect_column_values_to_be_of_type__map", column
+            new_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="_expect_column_values_to_be_of_type__map",
+                    kwargs={"column": column},
+                )
             )
             assert len(new_expectations) == 1
             old_config = self._expectation_suite.expectations[new_expectations[0]]
@@ -625,6 +750,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         type_,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -696,6 +823,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         type_,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -737,7 +866,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         # Since we've now received the default arguments *before* the expectation decorator, we need to
         # ensure we only pass what we actually received. Hence, we'll use kwargs
         # mostly=None,
-        # result_format=None, include_config=None, catch_exceptions=None, meta=None
+        # result_format = None,
+        # row_condition=None, condition_parser=None, include_config=None, catch_exceptions=None, meta=None
     ):
         """
         The pandas implementation of this expectation takes kwargs mostly, result_format, include_config,
@@ -773,14 +903,20 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
 
             # First, if there is an existing expectation of this type, delete it. Then change the one we created to be
             # of the proper expectation_type
-            existing_expectations = self.find_expectation_indexes(
-                "expect_column_values_to_be_in_type_list", column
+            existing_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="expect_column_values_to_be_in_type_list",
+                    kwargs={"column": column},
+                )
             )
             if len(existing_expectations) == 1:
                 self._expectation_suite.expectations.pop(existing_expectations[0])
 
-            new_expectations = self.find_expectation_indexes(
-                "_expect_column_values_to_be_in_type_list__aggregate", column
+            new_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="_expect_column_values_to_be_in_type_list__aggregate",
+                    kwargs={"column": column},
+                )
             )
             assert len(new_expectations) == 1
             old_config = self._expectation_suite.expectations[new_expectations[0]]
@@ -804,15 +940,21 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
 
             # First, if there is an existing expectation of this type, delete it. Then change the one we created to be
             # of the proper expectation_type
-            existing_expectations = self.find_expectation_indexes(
-                "expect_column_values_to_be_in_type_list", column
+            existing_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="expect_column_values_to_be_in_type_list",
+                    kwargs={"column": column},
+                )
             )
             if len(existing_expectations) == 1:
                 self._expectation_suite.expectations.pop(existing_expectations[0])
 
             # Now, rename the expectation we just added
-            new_expectations = self.find_expectation_indexes(
-                "_expect_column_values_to_be_in_type_list__map", column
+            new_expectations = self._expectation_suite.find_expectation_indexes(
+                ExpectationConfiguration(
+                    expectation_type="_expect_column_values_to_be_in_type_list__map",
+                    kwargs={"column": column},
+                )
             )
             assert len(new_expectations) == 1
             old_config = self._expectation_suite.expectations[new_expectations[0]]
@@ -833,6 +975,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         type_list,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -882,6 +1026,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         type_list,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -924,6 +1070,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         mostly=None,
         parse_strings_as_datetimes=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -931,6 +1079,7 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         if value_set is None:
             # Vacuously true
             return np.ones(len(column), dtype=np.bool_)
+
         if parse_strings_as_datetimes:
             parsed_value_set = self._parse_value_set(value_set)
         else:
@@ -947,6 +1096,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         mostly=None,
         parse_strings_as_datetimes=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -971,6 +1122,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         output_strftime_format=None,
         allow_cross_type_comparisons=None,
         mostly=None,
+        row_condition=None,
+        condition_parser=None,
         result_format=None,
         include_config=True,
         catch_exceptions=None,
@@ -1097,7 +1250,10 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column,
         strictly=None,
         parse_strings_as_datetimes=None,
+        output_strftime_format=None,
         mostly=None,
+        row_condition=None,
+        condition_parser=None,
         result_format=None,
         include_config=True,
         catch_exceptions=None,
@@ -1133,7 +1289,10 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column,
         strictly=None,
         parse_strings_as_datetimes=None,
+        output_strftime_format=None,
         mostly=None,
+        row_condition=None,
+        condition_parser=None,
         result_format=None,
         include_config=True,
         catch_exceptions=None,
@@ -1170,6 +1329,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         min_value=None,
         max_value=None,
         mostly=None,
+        row_condition=None,
+        condition_parser=None,
         result_format=None,
         include_config=True,
         catch_exceptions=None,
@@ -1212,6 +1373,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         value,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1226,6 +1389,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         regex,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1240,6 +1405,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         regex,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1255,6 +1422,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         match_on="any",
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1280,6 +1449,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         regex_list,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1299,6 +1470,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         strftime_format,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1310,7 +1483,7 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
                 datetime.strftime(datetime.now(), strftime_format), strftime_format
             )
         except ValueError as e:
-            raise ValueError("Unable to use provided strftime_format. " + e.message)
+            raise ValueError("Unable to use provided strftime_format. " + str(e))
 
         def is_parseable_by_format(val):
             try:
@@ -1332,6 +1505,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1358,6 +1533,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1379,6 +1556,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         json_schema,
         mostly=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1408,6 +1587,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         p_value=0.05,
         params=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1455,6 +1636,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         bootstrap_samples=None,
         bootstrap_sample_size=None,
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1564,6 +1747,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column_B,
         ignore_row_if="both_values_are_missing",
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1581,6 +1766,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         allow_cross_type_comparisons=None,
         ignore_row_if="both_values_are_missing",
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1611,6 +1798,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         value_pairs_set,
         ignore_row_if="both_values_are_missing",
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1645,6 +1834,8 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         column_list,
         ignore_row_if="all_values_are_missing",
         result_format=None,
+        row_condition=None,
+        condition_parser=None,
         include_config=True,
         catch_exceptions=None,
         meta=None,
@@ -1652,3 +1843,26 @@ class PandasDataset(MetaPandasDataset, pd.DataFrame):
         threshold = len(column_list.columns)
         # Do not dropna here, since we have separately dealt with na in decorator
         return column_list.nunique(dropna=False, axis=1) >= threshold
+
+    @DocInherit
+    @MetaPandasDataset.multicolumn_map_expectation
+    def expect_multicolumn_sum_to_equal(
+        self,
+        column_list,
+        sum_total,
+        result_format=None,
+        include_config=True,
+        catch_exceptions=None,
+        meta=None,
+    ):
+        """ Multi-Column Map Expectation
+
+        Expects that sum of all rows for a set of columns is equal to a specific value
+
+        Args:
+            column_list (List[str]): \
+                Set of columns to be checked
+            sum_total (int): \
+                expected sum of columns
+        """
+        return column_list.sum(axis=1) == sum_total
