@@ -9,7 +9,7 @@ logger = logging.getLogger(__name__)
 
 
 class BatchKwargsGenerator(object):
-    """
+    r"""
     BatchKwargsGenerators produce identifying information, called "batch_kwargs" that datasources
     can use to get individual batches of data. They add flexibility in how to obtain data
     such as with time-based partitioning, downsampling, or other techniques appropriate
@@ -68,7 +68,99 @@ class BatchKwargsGenerator(object):
              default:
                 class_name: QueryBatchKwargsGenerator
 
+--ge-feature-maturity-info--
 
+    id: batch_kwargs_generator_manual
+    title: Batch Kwargs Generator - Manual
+    icon:
+    short_description: Manually configure how files on a filesystem are presented as batches of data
+    description: Manually configure how files on a filesystem are presented as batches of data
+    how_to_guide_url:
+    maturity: Beta
+    maturity_details:
+        api_stability: Mostly Stable (key generator functionality will remain but batch API changes still possible)
+        implementation_completeness: Complete
+        unit_test_coverage: Complete
+        integration_infrastructure_test_coverage: N/A
+        documentation_completeness: Minimal
+        bug_risk: Moderate
+
+    id: batch_kwargs_generator_s3
+    title: Batch Kwargs Generator - S3
+    icon:
+    short_description: Present files on S3 as batches of data
+    description: Present files on S3 as batches of data for profiling and validation
+    how_to_guide_url:
+    maturity: Beta
+    maturity_details:
+        api_stability: Mostly Stable (expect changes in partitioning)
+        implementation_completeness: Partial
+        unit_test_coverage: Complete
+        integration_infrastructure_test_coverage: Complete
+        documentation_completeness: Minimal
+        bug_risk: Moderate
+
+    id: batch_kwargs_generator_glob_reader
+    title: Batch Kwargs Generator - Glob Reader
+    icon:
+    short_description: A configurable way to present files in a directory as batches of data
+    description: A configurable way to present files in a directory as batches of data
+    how_to_guide_url:
+    maturity: Beta
+    maturity_details:
+        api_stability: Mostly Stable (expect changes in partitioning)
+        implementation_completeness: Partial
+        unit_test_coverage: Complete
+        integration_infrastructure_test_coverage: N/A
+        documentation_completeness: Minimal
+        bug_risk: Moderate
+
+    id: batch_kwargs_generator_table
+    title: Batch Kwargs Generator - Table
+    icon:
+    short_description: Present database tables as batches of data
+    description: Present database tables as batches of data for validation and profiling
+    how_to_guide_url:
+    maturity: Beta
+    maturity_details:
+        api_stability: Unstable (no existing native support for "partitioning")
+        implementation_completeness: Minimal
+        unit_test_coverage: Partial
+        integration_infrastructure_test_coverage: Minimal
+        documentation_completeness: Partial
+        bug_risk: Low
+
+    id: batch_kwargs_generator_query
+    title: Batch Kwargs Generator - Query
+    icon:
+    short_description: Present the result sets of SQL queries as batches of data
+    description: Present the result sets of SQL queries as batches of data for validation and profiling
+    how_to_guide_url:
+    maturity: Beta
+    maturity_details:
+        api_stability: Unstable (expect changes in query template configuration and query storage)
+        implementation_completeness: Complete
+        unit_test_coverage: Partial
+        integration_infrastructure_test_coverage: Minimal
+        documentation_completeness: Partial
+        bug_risk: Moderate
+
+    id: batch_kwargs_generator_subdir_reader
+    title: Batch Kwargs Generator - Subdir Reader
+    icon:
+    short_description: Present the files in a directory as batches of data
+    description: Present the files in a directory as batches of data for profiling and validation.
+    how_to_guide_url:
+    maturity: Beta
+    maturity_details:
+        api_stability: Mostly Stable (new configuration options likely)
+        implementation_completeness: Partial
+        unit_test_coverage: Complete
+        integration_infrastructure_test_coverage: N/A
+        documentation_completeness: Minimal
+        bug_risk: Low
+
+--ge-feature-maturity-info--
     """
 
     _batch_kwargs_type = BatchKwargs
@@ -159,21 +251,18 @@ class BatchKwargsGenerator(object):
             self.reset_iterator(data_asset_name=data_asset_name, **kwargs)
             return self._data_asset_iterators[data_asset_name][0]
 
-    # TODO: deprecate generator_asset argument
-    def build_batch_kwargs(
-        self, name=None, partition_id=None, data_asset_name=None, **kwargs
-    ):
-        assert (
-            (name and not data_asset_name) or (not name and data_asset_name),
-            "Please provide either name or data_asset_name.",
-        )
-        if name:
+    def build_batch_kwargs(self, data_asset_name=None, partition_id=None, **kwargs):
+        if (not kwargs.get("name") and not data_asset_name) or (
+            kwargs.get("name") and data_asset_name
+        ):
+            raise ValueError("Please provide either name or data_asset_name.")
+        if kwargs.get("name"):
             warnings.warn(
                 "The 'name' argument will be deprecated and renamed to 'data_asset_name'. "
                 "Please update code accordingly.",
                 DeprecationWarning,
             )
-            data_asset_name = name
+            data_asset_name = kwargs.pop("name")
 
         """The key workhorse. Docs forthcoming."""
         if data_asset_name is not None:
@@ -204,7 +293,7 @@ class BatchKwargsGenerator(object):
         raise NotImplementedError
 
     # TODO: deprecate generator_asset argument
-    def yield_batch_kwargs(self, generator_asset=None, data_asset_name=None, **kwargs):
+    def yield_batch_kwargs(self, data_asset_name=None, generator_asset=None, **kwargs):
         assert (generator_asset and not data_asset_name) or (
             not generator_asset and data_asset_name
         ), "Please provide either generator_asset or data_asset_name."
